@Grapes([
  @Grab(group='commons-codec', module='commons-codec', version='1.9'),
  @Grab(group='org.codehaus.groovy.modules.http-builder', module='http-builder', version='0.7.1'),
  @Grab(group='oauth.signpost', module='signpost-core', version='1.2.1.2'),
  @Grab(group='oauth.signpost', module='signpost-commonshttp4', version='1.2.1.2')
])
 
import groovy.util.CliBuilder 
import groovy.xml.*
import groovyx.net.http.*
import javax.net.ssl.SSLException
import static groovyx.net.http.ContentType.*
import static groovyx.net.http.Method.*

import javax.xml.xpath.*

def xpath = XPathFactory.newInstance().newXPath()


def cli = new CliBuilder(usage:'getCustomObjects ')
cli.h('help')
cli.instance(args:1, argName: 'instance', 'instance name - directory to store results')
cli.url(args:1, argName: 'url', 'url of Axeda Machine Cloud')
cli.username(args:1, argName: 'username', 'username')
cli.password(args:1, argName: 'password', 'password')
cli.acceptBadSSL(args:0, argName: 'acceptBadSSL', 'Ignore any TLS validation issues')
options = cli.parse(args)

if ( options.h ) { 
  cli.usage() 
  System.exit(1)
} else if ( !options.instance ) { 
  cli.usage()
  System.exit(1)
}

def pageNumber = 1 
def pageSize   = 25 

def baseRestUrl = '/services/v2/rest/customObject/find' 

def queryXML = """ 
<CustomObjectCriteria xmlns="http://www.axeda.com/services/v2" pageSize="{{pageSize}}" pageNumber="{{pageNumber}}">
   <name>*</name>
</CustomObjectCriteria>
"""

def noMore = false
restUrl = options.url + baseRestUrl
println "Using URL: ${restUrl} with username: ${options.username}..." 

while ( !noMore )  { 
    def pageQuery = queryXML.replace('{{pageSize}}', pageSize.toString())
                            .replace('{{pageNumber}}', pageNumber.toString())

   
    def restClient = new RESTClient(restUrl)
    if ( options.acceptBadSSL ) {
        restClient.ignoreSSLIssues()
    }
    restClient.auth.basic options.username, options.password

    try { 
        restClient.request(POST, XML) {
            body = pageQuery 
            //requestContentType = 'application/xml' 
           
            response.success = { resp -> 
                def resp_text = resp.entity.content.text
 
                def slurped = new XmlSlurper().parseText(resp_text) 
                def co = slurped.customObjects.customObject 
                println "Size: " + co.size()

                if ( co.size() < pageSize ) { 
                    noMore = true
                } else { 
                    pageNumber++
                }
                co.each { currentCO -> 
                    writeFile(currentCO.name, currentCO.source)
                }
            }
 
        }
    } catch ( groovyx.net.http.HttpResponseException ex)  { 
        println ex.statusCode + ": " + ex.message
        if ( ex.statusCode == 401 ) {
            println "ERROR Authorization: try specifying -username and -password" 
        }
        break
    } catch ( javax.net.ssl.SSLException ex ) {
        println "ERROR SSLException: try specifying -acceptBadSSL"
        println ex.toString() 
        break
    } catch ( ex ) { 
        println ex.toString() 
        break
    } 
}

def writeFile(name, source) { 
    def destination = new File(options.instance) 
    if ( !destination.exists() ) {
        destination.mkdirs() 
    }

    def srcName = name.text().replace(':', '_')
    println "=========================\nWriting ${name}\n================="  
    def script = new File(destination, "${srcName}.groovy") 
    def writer = script.newWriter()
    source.writeTo(writer)
    writer.close()
}
